﻿using System.ComponentModel;
using nRoute.Components;

namespace nRoute.Samples.ReverseCommand.WPF.ViewModels
{
    public class MainPageViewModel : INotifyPropertyChanged
    {
        private const string MEH_MESSAGE = "Oky, mehhhh... mehhh...";
        private const string HELLNO_MESSAGE = "Oh well, you are not alone..";

        private string _message;
        private bool _movieIsPlaying;

        public MainPageViewModel()
        {
            SetupReverseCommands();
            SetupCommands();
        }

        #region Properties

        public string Message
        {
            get { return _message; }
            set
            {
                _message = value;
                PropertyChanged.Notify(() => Message);
            }
        }

        // Commands

        public ActionCommand HomeCommand { get; private set; }

        public ActionCommand<int> ResponseCommand { get; private set; }

        public ActionCommand PlayPauseCommand { get; private set; }

        // Reverse Commands

        public ActionCommand StartScreenReverseCommand { get; private set; }

        public ActionCommand TrailerScreenReverseCommand { get; private set; }

        public ActionCommand MessageScreenReverseCommand { get; private set; }

        public ActionCommand PlayReverseCommand { get; private set; }

        public ActionCommand PauseReverseCommand { get; private set; }

        #endregion

        #region Helpers

        void SetupReverseCommands()
        {
            StartScreenReverseCommand = new ActionCommand(() => { });
            TrailerScreenReverseCommand = new ActionCommand(() => { });
            MessageScreenReverseCommand = new ActionCommand(() => { });
            PlayReverseCommand = new ActionCommand(() => { });
            PauseReverseCommand = new ActionCommand(() => { });
        }

        void SetupCommands()
        {
            HomeCommand = new ActionCommand(() =>
            {
                if (_movieIsPlaying) SetMovieState(false);     // stop movie if playing
                StartScreenReverseCommand.Execute();           // got to start screen
            });

            ResponseCommand = new ActionCommand<int>((i) =>
            {
                if (i == 1)                                     // TRAILER REQUEST
                {
                    TrailerScreenReverseCommand.Execute();
                    SetMovieState(true);                        // start playing the trailer
                }
                else if (i == 0)                                // MEH RESPONSE
                {
                    Message = MEH_MESSAGE;
                    MessageScreenReverseCommand.Execute();
                }
                else if (i == -1)                               // HELL NO RESPONSE
                {
                    Message = HELLNO_MESSAGE;
                    MessageScreenReverseCommand.Execute();
                }
            });

            PlayPauseCommand = new ActionCommand(() => SetMovieState(!_movieIsPlaying));        // toggle the state
        }

        void SetMovieState(bool isPlaying)
        {
            if (isPlaying)
                PlayReverseCommand.Execute();
            else
                PauseReverseCommand.Execute();
            _movieIsPlaying = isPlaying;
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

    }
}
